@@ -1,7 +1,20 @@
 #ifndef TEST_RCLASPELL
-#ifndef lint
-static char rcsid[] = "@(#$Id: rclaspell.cpp,v 1.10 2007-12-13 06:58:21 dockes Exp $ (C) 2006 J.F.Dockes";
-#endif
+/* Copyright (C) 2006 J.F.Dockes
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the
+ *   Free Software Foundation, Inc.,
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ */
 #ifdef HAVE_CONFIG_H
 #include "autoconfig.h"
 #endif
@@ -54,14 +67,6 @@ static AspellApi aapi;
 	badnames += #NM + string(" ");					\
     }
 
-static const char *aspell_progs[] = {
-#ifdef ASPELL_PROG
-    ASPELL_PROG ,
-#endif
-    "/usr/local/bin/aspell",
-    "/usr/bin/aspell"
-};
-static const unsigned int naspellprogs = sizeof(aspell_progs) / sizeof(char*);
 static const char *aspell_lib_suffixes[] = {
   ".so",
   ".so.15",
@@ -124,12 +129,18 @@ bool Aspell::init(string &reason)
     }
 
     m_data = new AspellData;
-    for (unsigned int i = 0; i < naspellprogs; i++) {
-	if (access(aspell_progs[i], X_OK) == 0) {
-	    m_data->m_exec = aspell_progs[i];
-	    break;
-	}
+
+    const char *aspell_prog_from_env = getenv("ASPELL_PROG");
+    if (aspell_prog_from_env && access(aspell_prog_from_env, X_OK) == 0) {
+	m_data->m_exec = aspell_prog_from_env;
+#ifdef ASPELL_PROG
+    } else if (access(ASPELL_PROG, X_OK) == 0) {
+	m_data->m_exec = ASPELL_PROG;
+#endif // ASPELL_PROG
+    } else {
+	ExecCmd::which("aspell", m_data->m_exec);
     }
+
     if (m_data->m_exec.empty()) {
 	reason = "aspell program not found or not executable";
         deleteZ(m_data);
@@ -140,9 +151,8 @@ bool Aspell::init(string &reason)
     // be clever with ASPELL_PROG.
     vector<string> libdirs;
     libdirs.push_back(LIBDIR);
-#ifdef ASPELL_PROG
-    // The aspell library has to live under the same prefix as the 
-    // aspell program.
+    // If not in the standard place, the aspell library has to live
+    // under the same prefix as the aspell program.
     {
 	string aspellPrefix = path_getfather(path_getfather(m_data->m_exec));
 	// This would probably require some more tweaking on solaris/irix etc.
@@ -151,7 +161,6 @@ bool Aspell::init(string &reason)
 	if (libaspell != LIBDIR)
 	    libdirs.push_back(libaspell);
     }
-#endif
 
     reason = "Could not open shared library ";
     for (vector<string>::iterator it = libdirs.begin(); 
