@@ -1,4 +1,4 @@
-/*
+/* Copyright (C) 2004 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -16,7 +16,6 @@
  */
 #ifndef _INTERNFILE_H_INCLUDED_
 #define _INTERNFILE_H_INCLUDED_
-/* @(#$Id: internfile.h,v 1.21 2008-10-08 16:15:22 dockes Exp $  (C) 2004 J.F.Dockes */
 
 #include <string>
 #include <vector>
@@ -40,6 +39,19 @@ class Doc;
 
 struct stat;
 
+/** Storage for missing helper program info. We want to keep this out of the 
+ * FileInterner class, because the data will typically be accumulated by several
+ * FileInterner objects. Can't use static member either (because there
+ * may be several separate usages of the class which shouldn't mix
+ * their data).
+ */
+class FIMissingStore {
+public:
+    // Missing external programs
+    set<string>           m_missingExternal;
+    map<string, set<string> > m_typesForMissing;
+};
+
 /** 
  * A class to convert data from a datastore (file-system, firefox
  * history, etc.)  into possibly one or severaldocuments in internal
@@ -111,6 +123,11 @@ class FileInterner {
 
     ~FileInterner();
 
+    void setMissingStore(FIMissingStore *st)
+    {
+	m_missingdatap = st;
+    }
+
     /** 
      * Turn file or file part into Recoll document.
      * 
@@ -119,12 +136,12 @@ class FileInterner {
      * @param doc output document
      * @param ipath internal path. If set by caller, the specified subdoc will
      *  be returned. Else the next document according to current state will 
-     *  be returned, and ipath will be set on output.
+     *  be returned, and doc.ipath will be set on output.
      * @return FIError and FIDone are self-explanatory. If FIAgain is returned,
      *  this is a multi-document file, with more subdocs, and internfile() 
      *  should be called again to get the following one(s).
      */
-    Status internfile(Rcl::Doc& doc, string &ipath);
+    Status internfile(Rcl::Doc& doc, const string &ipath = "");
 
     /** Return the file's (top level object) mimetype (useful for 
      *  container files) 
@@ -170,11 +187,10 @@ class FileInterner {
                                       RclConfig *cnf, const Rcl::Doc& doc);
 
     const string& getReason() const {return m_reason;}
-    static void getMissingExternal(string& missing);
-    static void getMissingDescription(string& desc);
+    static void getMissingExternal(FIMissingStore *st, string& missing);
+    static void getMissingDescription(FIMissingStore *st, string& desc);
     bool ok() {return m_ok;}
 
-
  private:
     static const unsigned int MAXHANDLERS = 20;
     RclConfig             *m_cfg;
@@ -204,9 +220,7 @@ class FileInterner {
     vector<TempFile>       m_tempfiles;
     // Error data if any
     string                 m_reason;
-    // Missing external programs
-    static set<string>           o_missingExternal;
-    static map<string, set<string> > o_typesForMissing;
+    FIMissingStore        *m_missingdatap;
 
     // Pseudo-constructors
     void init(const string &fn, const struct stat *stp, 
