@@ -1,7 +1,4 @@
-#ifndef lint
-static char rcsid[] = "@(#$Id: main.cpp,v 1.73 2008-12-17 08:01:40 dockes Exp $ (C) 2005 J.F.Dockes";
-#endif
-/*
+/* Copyright (C) 2005 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -50,7 +47,16 @@ static char rcsid[] = "@(#$Id: main.cpp,v 1.73 2008-12-17 08:01:40 dockes Exp $
 #include "smallut.h"
 #include "recollq.h"
 
-RclConfig *rclconfig;
+RclConfig *theconfig;
+RclConfig *thestableconfig;
+PTMutexInit thestableconfiglock;
+
+void snapshotConfig()
+{
+    PTMutexLocker locker(thestableconfiglock);
+    thestableconfig = new RclConfig(*theconfig);
+}
+    
 Rcl::Db *rcldb;
 #ifdef RCL_USE_ASPELL
 Aspell *aspell;
@@ -66,7 +72,7 @@ void startManual(const string& helpindex)
 	mainWindow->startManual(helpindex);
 }
 
-bool maybeOpenDb(string &reason, bool force)
+bool maybeOpenDb(string &reason, bool force, bool *maindberror)
 {
     if (!rcldb) {
 	reason = "Internal error: db not created";
@@ -81,9 +87,13 @@ bool maybeOpenDb(string &reason, bool force)
 	LOGDEB(("main: adding [%s]\n", it->c_str()));
 	rcldb->addQueryDb(*it);
     }
-    if (!rcldb->isopen() && !rcldb->open(Rcl::Db::DbRO)) {
+    Rcl::Db::OpenError error;
+    if (!rcldb->isopen() && !rcldb->open(Rcl::Db::DbRO, &error)) {
 	reason = "Could not open database in " + 
-	    rclconfig->getDbDir() + " wait for indexing to complete?";
+	    theconfig->getDbDir() + " wait for indexing to complete?";
+	if (maindberror) {
+	    *maindberror = (error == Rcl::Db::DbOpenMainDb) ? true : false;
+	}
 	return false;
     }
     rcldb->setAbstractParams(-1, prefs.syntAbsLen, prefs.syntAbsCtx);
@@ -107,7 +117,8 @@ static void recollCleanup()
     rwSettings(true);
     LOGDEB2(("recollCleanup: closing database\n"));
     deleteZ(rcldb);
-    deleteZ(rclconfig);
+    deleteZ(theconfig);
+//    deleteZ(thestableconfig);
 #ifdef RCL_USE_ASPELL
     deleteZ(aspell);
 #endif
@@ -168,7 +179,7 @@ int main(int argc, char **argv)
 {
     for (int i = 0; i < argc; i++) {
 	if (!strcmp(argv[i], "-t")) {
-	    exit(recollq(&rclconfig, argc, argv));
+	    exit(recollq(&theconfig, argc, argv));
 	}
     }
 
@@ -221,17 +232,18 @@ int main(int argc, char **argv)
     app.installTranslator( &qt );
 
     string reason;
-    rclconfig = recollinit(recollCleanup, sigcleanup, reason, &a_config);
-    if (!rclconfig || !rclconfig->ok()) {
+    theconfig = recollinit(recollCleanup, sigcleanup, reason, &a_config);
+    if (!theconfig || !theconfig->ok()) {
 	QString msg = "Configuration problem: ";
 	msg += QString::fromUtf8(reason.c_str());
 	QMessageBox::critical(0, "Recoll",  msg);
 	exit(1);
     }
+    snapshotConfig();
     //    fprintf(stderr, "recollinit done\n");
 
     // Translations for Recoll
-    string translatdir = path_cat(rclconfig->getDatadir(), "translations");
+    string translatdir = path_cat(theconfig->getDatadir(), "translations");
     QTranslator translator(0);
     translator.load( QString("recoll_") + slang, translatdir.c_str() );
     app.installTranslator( &translator );
@@ -239,7 +251,7 @@ int main(int argc, char **argv)
     //    fprintf(stderr, "Translations installed\n");
 
 #ifdef RCL_USE_ASPELL
-    aspell = new Aspell(rclconfig);
+    aspell = new Aspell(theconfig);
     aspell->init(reason);
     if (!aspell || !aspell->ok()) {
 	LOGDEB(("Aspell speller creation failed %s\n", reason.c_str()));
@@ -247,7 +259,7 @@ int main(int argc, char **argv)
     }
 #endif
 
-    string historyfile = path_cat(rclconfig->getConfDir(), "history");
+    string historyfile = path_cat(theconfig->getConfDir(), "history");
     g_dynconf = new RclDynConf(historyfile);
     if (!g_dynconf || !g_dynconf->ok()) {
 	QString msg = app.translate("Main", "Configuration problem (dynconf");
@@ -269,7 +281,7 @@ int main(int argc, char **argv)
 	mainWindow->resize(s);
     }
 
-    string dbdir = rclconfig->getDbDir();
+    string dbdir = theconfig->getDbDir();
     if (dbdir.empty()) {
 	QMessageBox::critical(0, "Recoll",
 			      app.translate("Main", 
@@ -277,7 +289,7 @@ int main(int argc, char **argv)
 	exit(1);
     }
 
-    rcldb = new Rcl::Db(rclconfig);
+    rcldb = new Rcl::Db(theconfig);
 
     mainWindow->show();
     QTimer::singleShot(0, mainWindow, SLOT(initDbOpen()));
@@ -289,7 +301,7 @@ int main(int argc, char **argv)
 
     // Start the indexing thread. It will immediately go to sleep waiting for 
     // something to do.
-    start_idxthread(*rclconfig);
+    start_idxthread();
 
     mainWindow->sSearch->searchTypCMB->setCurrentIndex(prefs.ssearchTyp);
     mainWindow->sSearch->searchTypeChanged(prefs.ssearchTyp);
