@@ -1,7 +1,4 @@
-#ifndef lint
-static char rcsid[] = "@(#$Id: rclmain_w.cpp,v 1.57 2008-10-13 07:57:12 dockes Exp $ (C) 2005 J.F.Dockes";
-#endif
-/*
+/* Copyright (C) 2005 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -100,6 +97,7 @@ void RclMain::init()
     spellform = 0;
     m_idxStatusAck = false;
     periodictimer = new QTimer(this);
+    m_periodicToggle = 0;
 
     // At least some versions of qt4 don't display the status bar if
     // it's not created here.
@@ -115,8 +113,10 @@ void RclMain::init()
     g_stringNoStem = tr("(no stemming)");
     g_stringAllStem = tr("(all languages)");
     m_idNoStem = preferencesMenu->addAction(g_stringNoStem);
+    m_idNoStem->setCheckable(true);
     m_stemLangToId[g_stringNoStem] = m_idNoStem;
     m_idAllStem = preferencesMenu->addAction(g_stringAllStem);
+    m_idAllStem->setCheckable(true);
     m_stemLangToId[g_stringAllStem] = m_idAllStem;
 
     // Can't get the stemming languages from the db at this stage as
@@ -125,7 +125,7 @@ void RclMain::init()
     // instead
     string slangs;
     list<string> langs;
-    if (rclconfig->getConfParam("indexstemminglanguages", slangs)) {
+    if (theconfig->getConfParam("indexstemminglanguages", slangs)) {
 	stringToStrings(slangs, langs);
     } else {
 	QMessageBox::warning(0, "Recoll", 
@@ -137,6 +137,7 @@ void RclMain::init()
 	 it != langs.end(); it++) {
 	QString qlang = QString::fromAscii(it->c_str(), it->length());
 	id = preferencesMenu->addAction(qlang);
+	id->setCheckable(true);
 	m_stemLangToId[qlang] = id;
 	if (prefs.queryStemLang == qlang) {
 	    curid = id;
@@ -166,7 +167,7 @@ void RclMain::init()
     connect(bgrp, SIGNAL(buttonClicked(int)), this, SLOT(catgFilter(int)));
     allRDB->setChecked(true);
     list<string> cats;
-    rclconfig->getMimeCategories(cats);
+    theconfig->getMimeCategories(cats);
     // Text for button 0 is not used. Next statement just avoids unused
     // variable compiler warning for catg_strings
     m_catgbutvec.push_back(catg_strings[0]);
@@ -238,7 +239,9 @@ void RclMain::init()
 	    this, SLOT(showUIPrefs()));
     connect(extIdxAction, SIGNAL(activated()), 
 	    this, SLOT(showExtIdxDialog()));
-	    
+    connect(this, SIGNAL(applyFiltSortData()), 
+	    this, SLOT(onResultsChanged()));
+
     if (prefs.catgToolBar && catgCMB)
 	connect(catgCMB, SIGNAL(activated(int)), 
 		this, SLOT(catgFilter(int)));
@@ -257,12 +260,24 @@ void RclMain::init()
     connect(this, SIGNAL(sortDataChanged(DocSeqSortSpec)), 
 	    restable, SLOT(onSortDataChanged(DocSeqSortSpec)));
 
-    connect(restable->getModel(), SIGNAL(sortColumnChanged(DocSeqSortSpec)),
-	    this, SLOT(onResTableSortBy(DocSeqSortSpec)));
+    connect(restable->getModel(), SIGNAL(sortDataChanged(DocSeqSortSpec)),
+	    this, SLOT(onSortDataChanged(DocSeqSortSpec)));
     connect(restable, SIGNAL(docEditClicked(Rcl::Doc)), 
 	    this, SLOT(startNativeViewer(Rcl::Doc)));
     connect(restable, SIGNAL(docPreviewClicked(int, Rcl::Doc, int)), 
 	    this, SLOT(startPreview(int, Rcl::Doc, int)));
+    connect(restable, SIGNAL(docEditClicked(Rcl::Doc)), 
+	    this, SLOT(startNativeViewer(Rcl::Doc)));
+    connect(restable, SIGNAL(docExpand(Rcl::Doc)), 
+	    this, SLOT(docExpand(Rcl::Doc)));
+    connect(restable, SIGNAL(docEditClicked(Rcl::Doc)), 
+	    this, SLOT(startNativeViewer(Rcl::Doc)));
+    connect(restable, SIGNAL(previewRequested(Rcl::Doc)), 
+	    this, SLOT(startPreview(Rcl::Doc)));
+    connect(restable, SIGNAL(editRequested(Rcl::Doc)), 
+	    this, SLOT(startNativeViewer(Rcl::Doc)));
+    connect(restable, SIGNAL(docSaveToFileClicked(Rcl::Doc)), 
+	    this, SLOT(saveDocToFile(Rcl::Doc)));
 
     connect(this, SIGNAL(docSourceChanged(RefCntr<DocSequence>)),
 	    reslist, SLOT(setDocSource(RefCntr<DocSequence>)));
@@ -305,11 +320,10 @@ void RclMain::init()
 	    this, SLOT(showQueryDetails()));
 
     if (prefs.keepSort && prefs.sortActive) {
-	if (prefs.sortDesc) 
-	    actionSortByDateDesc->setChecked(true);
-	else
-	    actionSortByDateAsc->setChecked(true);
-	onSortCtlChanged();
+	m_sortspec.field = (const char *)prefs.sortField.toUtf8();
+	m_sortspec.desc = prefs.sortDesc;
+	onSortDataChanged(m_sortspec);
+	emit sortDataChanged(m_sortspec);
     }
 
     // Start timer on a slow period (used for checking ^C). Will be
@@ -330,25 +344,31 @@ void RclMain::initDbOpen()
     bool needindexconfig = false;
     bool nodb = false;
     string reason;
-    if (!maybeOpenDb(reason)) {
+    bool maindberror;
+    if (!maybeOpenDb(reason, true, &maindberror)) {
         nodb = true;
-	switch (QMessageBox::
-		question
-		(this, "Recoll",
-		 qApp->translate("Main", "Could not open database in ") +
-		 QString::fromLocal8Bit(rclconfig->getDbDir().c_str()) +
-		 qApp->translate("Main", 
-			       ".\n"
-			       "Click Cancel if you want to edit the configuration file before indexing starts, or Ok to let it proceed."),
-		 "Ok", "Cancel", 0,   0)) {
+	if (maindberror) {
+	    switch (QMessageBox::
+		    question
+		    (this, "Recoll",
+		     qApp->translate("Main", "Could not open database in ") +
+		     QString::fromLocal8Bit(theconfig->getDbDir().c_str()) +
+		     qApp->translate("Main", 
+				     ".\n"
+				     "Click Cancel if you want to edit the configuration file before indexing starts, or Ok to let it proceed."),
+		     "Ok", "Cancel", 0,   0)) {
 
-	case 0: // Ok: indexing is going to start.
-	    start_indexing(true);
-	    break;
+	    case 0: // Ok: indexing is going to start.
+		start_indexing(true);
+		break;
 
-	case 1: // Cancel
-	    needindexconfig = true;
-	    break;
+	    case 1: // Cancel
+		needindexconfig = true;
+		break;
+	    }
+	} else {
+	    QMessageBox::warning(0, "Recoll", 
+				 tr("Could not open external index. Db not open. Check external indexes list."));
 	}
     }
 
@@ -373,7 +393,7 @@ void RclMain::focusToSearch()
 
 void RclMain::setStemLang(QAction *id)
 {
-    LOGDEB(("RclMain::setStemLang(%d)\n", id));
+    LOGDEB(("RclMain::setStemLang(%p)\n", id));
     // Check that the menu entry is for a stemming language change
     // (might also be "show prefs" etc.
     bool isLangId = false;
@@ -478,7 +498,6 @@ void RclMain::fileExit()
 // indexing thread and a possible need to exit
 void RclMain::periodic100()
 {
-    static int toggle = 0;
     // Check if indexing thread done
     if (idxthread_getStatus() != IDXTS_NULL) {
 	// Indexing is stopped
@@ -513,7 +532,7 @@ void RclMain::periodic100()
 	fileToggleIndexingAction->setEnabled(TRUE);
         periodictimer->setInterval(100);
 	// The toggle thing is for the status to flash
-	if (toggle < 9) {
+	if (m_periodicToggle < 9) {
 	    QString msg = tr("Indexing in progress: ");
 	    DbIxStatus status = idxthread_idxStatus();
 	    QString phs;
@@ -534,17 +553,17 @@ void RclMain::periodic100()
 		msg += QString::fromAscii(cnts) + " ";
 	    }
 	    string mf;int ecnt = 0;
-	    string fcharset = rclconfig->getDefCharset(true);
+	    string fcharset = theconfig->getDefCharset(true);
 	    if (!transcode(status.fn, mf, fcharset, "UTF-8", &ecnt) || ecnt) {
 		mf = url_encode(status.fn, 0);
 	    }
 	    msg += QString::fromUtf8(mf.c_str());
 	    statusBar()->showMessage(msg, 4000);
-	} else if (toggle == 9) {
+	} else if (m_periodicToggle == 9) {
 	    statusBar()->showMessage("");
 	}
-	if (++toggle >= 10)
-	    toggle = 0;
+	if (++m_periodicToggle >= 10)
+	    m_periodicToggle = 0;
     }
     if (recollNeedsExit)
 	fileExit();
@@ -588,7 +607,7 @@ void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata)
 
     string stemLang = (const char *)prefs.queryStemLang.toAscii();
     if (stemLang == "ALL") {
-	rclconfig->getConfParam("indexstemminglanguages", stemLang);
+	theconfig->getConfParam("indexstemminglanguages", stemLang);
     }
     sdata->setStemlang(stemLang);
 
@@ -618,6 +637,22 @@ void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata)
     QApplication::restoreOverrideCursor();
 }
 
+void RclMain::onResultsChanged()
+{
+    if (m_source.isNotNull()) {
+	int cnt = m_source->getResCnt();
+	QString msg;
+	if (cnt > 0) {
+	    QString str;
+	    msg = tr("Result count (est.)") + ": " + 
+		str.setNum(cnt);
+	} else {
+	    msg = tr("No results found");
+	}
+	statusBar()->showMessage(msg, 0);
+    }
+}
+
 void RclMain::resetSearch()
 {
     emit searchReset();
@@ -662,7 +697,7 @@ void RclMain::showIndexConfig()
 {
     LOGDEB(("showIndexConfig()\n"));
     if (indexConfig == 0) {
-	indexConfig = new ConfIndexW(0, rclconfig);
+	indexConfig = new ConfIndexW(0, theconfig);
 	connect(new QShortcut(quitKeySeq, indexConfig), SIGNAL (activated()), 
 		this, SLOT (fileExit()));
     } else {
@@ -714,7 +749,7 @@ void RclMain::showAboutDialog()
 
 void RclMain::showMissingHelpers()
 {
-    string miss = rclconfig->getMissingHelperDesc();
+    string miss = theconfig->getMissingHelperDesc();
     QString msg = tr("External applications/commands needed and not found "
 		     "for indexing your file types:\n\n");
     if (!miss.empty()) {
@@ -879,23 +914,26 @@ void RclMain::onSortCtlChanged()
 	m_sortspec.desc = false;
 	prefs.sortActive = true;
 	prefs.sortDesc = false;
+	prefs.sortField = "mtime";
     } else if (actionSortByDateDesc->isChecked()) {
 	m_sortspec.field = "mtime";
 	m_sortspec.desc = true;
 	prefs.sortActive = true;
 	prefs.sortDesc = true;
+	prefs.sortField = "mtime";
     } else {
 	prefs.sortActive = prefs.sortDesc = false;
+	prefs.sortField = "";
     }
-    LOGDEB(("RclMain::onCtlDataChanged(): emitting change signals\n"));
     if (m_source.isNotNull())
 	m_source->setSortSpec(m_sortspec);
     emit sortDataChanged(m_sortspec);
     emit applyFiltSortData();
 }
 
-void RclMain::onResTableSortBy(DocSeqSortSpec spec)
+void RclMain::onSortDataChanged(DocSeqSortSpec spec)
 {
+    LOGDEB(("RclMain::onSortDataChanged\n"));
     m_sortspecnochange = true;
     if (spec.field.compare("mtime")) {
 	actionSortByDateDesc->setChecked(false);
@@ -908,6 +946,11 @@ void RclMain::onResTableSortBy(DocSeqSortSpec spec)
     if (m_source.isNotNull())
 	m_source->setSortSpec(spec);
     m_sortspec = spec;
+
+    prefs.sortField = QString::fromUtf8(spec.field.c_str());
+    prefs.sortDesc = spec.desc;
+    prefs.sortActive = !spec.field.empty();
+
     emit applyFiltSortData();
 }
 
@@ -954,7 +997,7 @@ void RclMain::saveDocToFile(Rcl::Doc doc)
 	    );
     string tofile((const char *)s.toLocal8Bit());
     TempFile temp; // not used
-    if (!FileInterner::idocToFile(temp, tofile, rclconfig, doc)) {
+    if (!FileInterner::idocToFile(temp, tofile, theconfig, doc)) {
 	QMessageBox::warning(0, "Recoll",
 			     tr("Cannot extract document or create "
 				"temporary file"));
@@ -1004,13 +1047,13 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
     // Look for appropriate viewer
     string cmdplusattr;
     if (prefs.useDesktopOpen) {
-	cmdplusattr = rclconfig->getMimeViewerDef("application/x-all", "");
+	cmdplusattr = theconfig->getMimeViewerDef("application/x-all", "");
     } else {
         string apptag;
         map<string,string>::const_iterator it;
         if ((it = doc.meta.find(Rcl::Doc::keyapptg)) != doc.meta.end())
             apptag = it->second;
-	cmdplusattr = rclconfig->getMimeViewerDef(doc.mimetype, apptag);
+	cmdplusattr = theconfig->getMimeViewerDef(doc.mimetype, apptag);
     }
 
     if (cmdplusattr.empty()) {
@@ -1023,7 +1066,7 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
     // Extract possible viewer attributes
     ConfSimple attrs;
     string cmd;
-    rclconfig->valueSplitAttributes(cmdplusattr, cmd, attrs);
+    theconfig->valueSplitAttributes(cmdplusattr, cmd, attrs);
     bool ignoreipath = false;
     if (attrs.get("ignoreipath", cmdplusattr))
         ignoreipath = stringToBool(cmdplusattr);
@@ -1043,7 +1086,7 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
     // directory
     string cmdpath;
     if (!ExecCmd::which(lcmd.front(), cmdpath)) {
-	cmdpath = rclconfig->findFilter(lcmd.front());
+	cmdpath = theconfig->findFilter(lcmd.front());
 	// findFilter returns its input param if the filter is not in
 	// the normal places. As we already looked in the path, we
 	// have no use for a simple command name here (as opposed to
@@ -1098,10 +1141,10 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 
     // If the command wants a file but this is not a file url, or
     // there is an ipath that it won't understand, we need a temp file:
-    rclconfig->setKeyDir(path_getfather(fn));
+    theconfig->setKeyDir(path_getfather(fn));
     if ((wantsfile && fn.empty()) || (!wantsipath && !doc.ipath.empty())) {
 	TempFile temp;
-	if (!FileInterner::idocToFile(temp, string(), rclconfig, doc)) {
+	if (!FileInterner::idocToFile(temp, string(), theconfig, doc)) {
 	    QMessageBox::warning(0, "Recoll",
 				 tr("Cannot extract document or create "
 				    "temporary file"));
@@ -1115,7 +1158,7 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 
     // If using an actual file, check that it exists, and if it is
     // compressed, we may need an uncompressed version
-    if (!fn.empty() && rclconfig->mimeViewerNeedsUncomp(doc.mimetype)) {
+    if (!fn.empty() && theconfig->mimeViewerNeedsUncomp(doc.mimetype)) {
         if (access(fn.c_str(), R_OK) != 0) {
             QMessageBox::warning(0, "Recoll", 
                                  tr("Can't access file: ") + 
@@ -1123,8 +1166,8 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
             return;
         }
         TempFile temp;
-        if (FileInterner::isCompressed(fn, rclconfig)) {
-            if (!FileInterner::maybeUncompressToTemp(temp, fn, rclconfig,  
+        if (FileInterner::isCompressed(fn, theconfig)) {
+            if (!FileInterner::maybeUncompressToTemp(temp, fn, theconfig,  
                                                      doc)) {
                 QMessageBox::warning(0, "Recoll", 
                                      tr("Can't uncompress file: ") + 
@@ -1178,7 +1221,7 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
     ncmd += " &";
     QStatusBar *stb = statusBar();
     if (stb) {
-	string fcharset = rclconfig->getDefCharset(true);
+	string fcharset = theconfig->getDefCharset(true);
 	string prcmd;
 	transcode(ncmd, prcmd, fcharset, "UTF-8");
 	QString msg = tr("Executing: [") + 
@@ -1204,7 +1247,7 @@ void RclMain::startManual(const string& index)
 {
     Rcl::Doc doc;
     doc.url = "file://";
-    doc.url = path_cat(doc.url, rclconfig->getDatadir());
+    doc.url = path_cat(doc.url, theconfig->getDatadir());
     doc.url = path_cat(doc.url, "doc");
     doc.url = path_cat(doc.url, "usermanual.html");
     LOGDEB(("RclMain::startManual: help index is %s\n", 
@@ -1352,7 +1395,7 @@ void RclMain::catgFilter(int id)
     if (id != 0)  {
 	string catg = m_catgbutvec[id];
 	list<string> tps;
-	rclconfig->getMimeCatTypes(catg, tps);
+	theconfig->getMimeCatTypes(catg, tps);
 	for (list<string>::const_iterator it = tps.begin();
 	     it != tps.end(); it++) 
 	    m_filtspec.orCrit(DocSeqFiltSpec::DSFS_MIMETYPE, *it);
