@@ -1,6 +1,19 @@
-#ifndef lint
-static char rcsid[] = "@(#$Id: reslist.cpp,v 1.52 2008-12-17 15:12:08 dockes Exp $ (C) 2005 J.F.Dockes";
-#endif
+/* Copyright (C) 2005 J.F.Dockes
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the
+ *   Free Software Foundation, Inc.,
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ */
 
 #include "autoconfig.h"
 
@@ -145,7 +158,7 @@ string QtGuiResListPager::pageTop()
 string QtGuiResListPager::iconPath(const string& mtype)
 {
     string iconpath;
-    rclconfig->getMimeIconName(mtype, &iconpath);
+    theconfig->getMimeIconName(mtype, &iconpath);
     return iconpath;
 }
 
@@ -154,7 +167,7 @@ void QtGuiResListPager::suggest(const vector<string>uterms, vector<string>&sugg)
     sugg.clear();
 #ifdef RCL_USE_ASPELL
     bool noaspell = false;
-    rclconfig->getConfParam("noaspell", &noaspell);
+    theconfig->getConfParam("noaspell", &noaspell);
     if (noaspell)
         return;
     if (!aspell) {
@@ -511,7 +524,7 @@ void ResList::displayPage()
 {
     m_pageParaToReldocnums.clear();
     clear();
-    m_pager->displayPage(rclconfig);
+    m_pager->displayPage(theconfig);
     LOGDEB0(("ResList::resultPageNext: hasNext %d hasPrev %d\n",
 	    m_pager->hasPrev(), m_pager->hasNext()));
     emit prevPageAvailable(m_pager->hasPrev());
