@@ -1,7 +1,4 @@
-#ifndef lint
-static char rcsid[] = "@(#$Id: spell_w.cpp,v 1.11 2007-02-19 16:28:05 dockes Exp $ (C) 2005 J.F.Dockes";
-#endif
-/*
+/* Copyright (C) 2005 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -53,7 +50,7 @@ void SpellW::init()
     /*2*/expTypeCMB->addItem(tr("Stem expansion"));
 #ifdef RCL_USE_ASPELL
     bool noaspell = false;
-    rclconfig->getConfParam("noaspell", &noaspell);
+    theconfig->getConfParam("noaspell", &noaspell);
     if (!noaspell)
 	/*3*/expTypeCMB->addItem(tr("Spelling/Phonetic"));
 #endif
