@@ -1,7 +1,4 @@
-#ifndef lint
-static char rcsid[] = "@(#$Id: uiprefs_w.cpp,v 1.26 2008-10-03 08:09:36 dockes Exp $ (C) 2005 J.F.Dockes";
-#endif
-/*
+/* Copyright (C) 2005 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -375,7 +372,7 @@ void UIPrefsDialog::addExtraDbPB_clicked()
     }
     struct stat st1, st2;
     stat(dbdir.c_str(), &st1);
-    string rcldbdir = rclconfig->getDbDir();
+    string rcldbdir = theconfig->getDbDir();
     stat(rcldbdir.c_str(), &st2);
     path_catslash(rcldbdir);
 
