@@ -1,7 +1,4 @@
-#ifndef lint
-static char rcsid[] = "@(#$Id: plaintorich.cpp,v 1.1 2008-11-19 12:19:40 dockes Exp $ (C) 2005 J.F.Dockes";
-#endif
-/*
+/* Copyright (C) 2005 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -315,7 +312,7 @@ bool PlainToRich::plaintorich(const string& in,
     const vector<vector<string> >& groups(hdata.groups);
     const vector<int>& slacks(hdata.gslks);
 
-    if (DebugLog::getdbl()->getlevel() >= DEBDEB0) {
+    if (0 && DebugLog::getdbl()->getlevel() >= DEBDEB0) {
 	LOGDEB0(("plaintorich: terms: \n"));
 	string sterms = vecStringToString(terms);
 	LOGDEB0(("  %s\n", sterms.c_str()));
@@ -338,7 +335,7 @@ bool PlainToRich::plaintorich(const string& in,
     // Note: the splitter returns the term locations in byte, not
     // character, offsets.
     splitter.text_to_words(in);
-    LOGDEB0(("plaintorich: split done %d mS\n", chron.millis()));
+    LOGDEB2(("plaintorich: split done %d mS\n", chron.millis()));
 
     // Compute the positions for NEAR and PHRASE groups.
     splitter.matchGroups();
@@ -435,6 +432,8 @@ bool PlainToRich::plaintorich(const string& in,
             if (eol > 2)
                 eol = 2;
             while (eol) {
+		if (!m_inputhtml && m_eolbr)
+		    *olit += "<br>";
                 *olit += "\n";
                 eol--;
             }
@@ -498,6 +497,6 @@ bool PlainToRich::plaintorich(const string& in,
 	fclose(fp);
     }
 #endif
-    LOGDEB0(("plaintorich: done %d mS\n", chron.millis()));
+    LOGDEB2(("plaintorich: done %d mS\n", chron.millis()));
     return true;
 }
