@@ -1,4 +1,4 @@
-/*
+/* Copyright (C) 2004 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -16,7 +16,6 @@
  */
 #ifndef _PLAINTORICH_H_INCLUDED_
 #define _PLAINTORICH_H_INCLUDED_
-/* @(#$Id: plaintorich.h,v 1.2 2008-12-16 14:20:10 dockes Exp $  (C) 2004 J.F.Dockes */
 
 #include <string>
 #include <list>
@@ -89,6 +88,8 @@ public:
 protected:
     static const string snull;
     bool m_inputhtml;
+    // Use <br> to break plain text lines (else caller has used a <pre> tag)
+    bool m_eolbr; 
 };
 
 #endif /* _PLAINTORICH_H_INCLUDED_ */
