@@ -1,7 +1,4 @@
-#ifndef lint
-static char rcsid[] = "@(#$Id: rcldb.cpp,v 1.154 2008-12-17 16:19:58 dockes Exp $ (C) 2004 J.F.Dockes";
-#endif
-/*
+/* Copyright (C) 2004 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -53,6 +50,7 @@ using namespace std;
 #include "rclquery_p.h"
 #include "md5.h"
 #include "rclversion.h"
+#include "cancelcheck.h"
 
 #ifndef MAX
 #define MAX(A,B) (A>B?A:B)
@@ -83,7 +81,7 @@ string version_string(){
 
 // Synthetic abstract marker (to discriminate from abstract actually
 // found in document)
-const static string rclSyntAbs("?!#@");
+static const string rclSyntAbs("?!#@");
 
 // Compute the unique term used to link documents to their origin. 
 // "Q" + external udi
@@ -221,7 +219,7 @@ static void listList(const string& what, const list<string>&l)
 string Db::Native::makeAbstract(Xapian::docid docid, Query *query)
 {
     Chrono chron;
-    LOGDEB0(("makeAbstract:%d: maxlen %d wWidth %d\n", chron.ms(),
+    LOGDEB2(("makeAbstract:%d: maxlen %d wWidth %d\n", chron.ms(),
 	     m_rcldb->m_synthAbsLen, m_rcldb->m_synthAbsWordCtxLen));
 
     list<string> terms;
@@ -500,7 +498,7 @@ string Db::Native::makeAbstract(Xapian::docid docid, Query *query)
     if (!abstract.compare("... "))
 	abstract.clear();
 
-    LOGDEB0(("makeAbtract: done in %d mS\n", chron.millis()));
+    LOGDEB2(("makeAbtract: done in %d mS\n", chron.millis()));
     return abstract;
 }
 
@@ -509,7 +507,7 @@ string Db::Native::makeAbstract(Xapian::docid docid, Query *query)
 Db::Db(RclConfig *cfp)
     : m_ndb(0), m_config(cfp), m_idxAbsTruncLen(250), m_synthAbsLen(250),
       m_synthAbsWordCtxLen(4), m_flushMb(-1), 
-      m_curtxtsz(0), m_flushtxtsz(0), m_occtxtsz(0),
+      m_curtxtsz(0), m_flushtxtsz(0), m_occtxtsz(0), m_occFirstCheck(1),
       m_maxFsOccupPc(0), m_mode(Db::DbRO)
 {
     m_ndb = new Native(this);
@@ -536,8 +534,11 @@ list<string> Db::getStemmerNames()
     return res;
 }
 
-bool Db::open(OpenMode mode)
+bool Db::open(OpenMode mode, OpenError *error)
 {
+    if (error)
+	*error = DbOpenMainDb;
+
     if (m_ndb == 0 || m_config == 0) {
 	m_reason = "Null configuration or Xapian Db";
 	return false;
@@ -588,12 +589,17 @@ bool Db::open(OpenMode mode)
 	    m_ndb->xrdb = Xapian::Database(dir);
 	    for (list<string>::iterator it = m_extraDbs.begin();
 		 it != m_extraDbs.end(); it++) {
+		if (error)
+		    *error = DbOpenExtraDb;
 		LOGDEB(("Db::Open: adding query db [%s]\n", it->c_str()));
-                // Used to be non-fatal (1.13 and older) but I can't see why
+                // An error here used to be non-fatal (1.13 and older)
+                // but I can't see why
                 m_ndb->xrdb.add_database(Xapian::Database(*it));
 	    }
 	    break;
 	}
+	if (error)
+	    *error = DbOpenMainDb;
 
 	// Check index format version. Must not try to check a just created or
 	// truncated db
@@ -610,6 +616,8 @@ bool Db::open(OpenMode mode)
 	m_mode = mode;
 	m_ndb->m_isopen = true;
 	m_basedir = dir;
+	if (error)
+	    *error = DbOpenNoError;
 	return true;
     } XCATCHERROR(ermsg);
 
@@ -751,12 +759,15 @@ bool Db::isopen()
 // reason (old config not updated ?). We use it only if the config
 // translation fails. Also we add in there fields which should be
 // indexed with no prefix (ie: abstract)
-bool Db::fieldToPrefix(const string& fld, string &pfx)
-{
-    // This is the default table. We prefer the data from rclconfig if 
-    // available
-    static map<string, string> fldToPrefs;
-    if (fldToPrefs.empty()) {
+
+// Default table. We prefer the data from rclconfig if available. Note
+// that it is logically const after initialization. This would be
+// simpler with c0xx initializer lists.
+static map<string, string> fldToPrefs;
+class InitFldToPrefs {
+public:
+    InitFldToPrefs() 
+    {
 	fldToPrefs[Doc::keyabs] = string();
 	fldToPrefs["ext"] = "XE";
 	fldToPrefs[Doc::keyfn] = "XSFN";
@@ -778,7 +789,11 @@ bool Db::fieldToPrefix(const string& fld, string &pfx)
         fldToPrefs["xapyearmon"] = "M";
         fldToPrefs["xapdate"] = "D";
     }
+};
+static InitFldToPrefs IFTP;
 
+bool Db::fieldToPrefix(const string& fld, string &pfx)
+{
     if (m_config && m_config->getFieldPrefix(fld, pfx))
 	return true;
 
@@ -879,12 +894,12 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi,
 	     udi.c_str(), parent_udi.c_str()));
     if (m_ndb == 0)
 	return false;
-    static int first = 1;
     // Check file system full every mbyte of indexed text.
-    if (m_maxFsOccupPc > 0 && (first || (m_curtxtsz - m_occtxtsz) / MB >= 1)) {
+    if (m_maxFsOccupPc > 0 && 
+	(m_occFirstCheck || (m_curtxtsz - m_occtxtsz) / MB >= 1)) {
 	LOGDEB(("Db::add: checking file system usage\n"));
 	int pc;
-	first = 0;
+	m_occFirstCheck = 0;
 	if (fsocc(m_basedir, &pc) && pc >= m_maxFsOccupPc) {
 	    LOGERR(("Db::add: stop indexing: file system "
 		     "%d%% full > max %d%%\n", pc, m_maxFsOccupPc));
@@ -1318,8 +1333,17 @@ bool Db::purge()
 
     // Walk the document array and delete any xapian document whose
     // flag is not set (we did not see its source during indexing).
+    int purgecount = 0;
     for (Xapian::docid docid = 1; docid < updated.size(); ++docid) {
 	if (!updated[docid]) {
+	    if ((purgecount+1) % 100 == 0) {
+		try {
+		    CancelCheck::instance().checkCancel();
+		} catch(CancelExcept) {
+		    LOGINFO(("Db::purge: partially cancelled\n"));
+		    break;
+		}
+	    }
 	    try {
 		m_ndb->xwdb.delete_document(docid);
 		LOGDEB(("Db::purge: deleted document #%d\n", docid));
@@ -1330,6 +1354,7 @@ bool Db::purge()
 	    } catch (...) {
 		LOGERR(("Db::purge: document #%d: unknown error\n", docid));
 	    }
+	    purgecount++;
 	}
     }
 
