@@ -1,6 +1,19 @@
-#ifndef lint
-static char rcsid[] = "@(#$Id: rclquery.cpp,v 1.11 2008-12-19 09:55:36 dockes Exp $ (C) 2008 J.F.Dockes";
-#endif
+/* Copyright (C) 2008 J.F.Dockes
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the
+ *   Free Software Foundation, Inc.,
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -90,7 +103,7 @@ private:
 
 Query::Query(Db *db)
     : m_nq(new Native(this)), m_db(db), m_sorter(0), m_sortAscending(true),
-      m_collapseDuplicates(false)
+      m_collapseDuplicates(false), m_resCnt(-1)
 {
 }
 
@@ -136,6 +149,7 @@ bool Query::setQuery(RefCntr<SearchData> sdata)
 	LOGERR(("Query::setQuery: not initialised!\n"));
 	return false;
     }
+    m_resCnt = -1;
     m_reason.erase();
 
     m_nq->clear();
@@ -258,23 +272,25 @@ int Query::getResCnt()
 	LOGERR(("Query::getResCnt: no query opened\n"));
 	return -1;
     }
+    if (m_resCnt >= 0)
+	return m_resCnt;
 
-    int ret = -1;
+    m_resCnt = -1;
     if (m_nq->xmset.size() <= 0) {
         Chrono chron;
 
         XAPTRY(m_nq->xmset = 
-               m_nq->xenquire->get_mset(0, qquantum, (const Xapian::RSet *)0);
-               ret = m_nq->xmset.get_matches_lower_bound(),
+               m_nq->xenquire->get_mset(0, qquantum, 1000);
+               m_resCnt = m_nq->xmset.get_matches_lower_bound(),
                m_db->m_ndb->xrdb, m_reason);
 
         LOGDEB(("Query::getResCnt: %d mS\n", chron.millis()));
 	if (!m_reason.empty())
 	    LOGERR(("xenquire->get_mset: exception: %s\n", m_reason.c_str()));
     } else {
-        ret = m_nq->xmset.get_matches_lower_bound();
+        m_resCnt = m_nq->xmset.get_matches_lower_bound();
     }
-    return ret;
+    return m_resCnt;
 }
 
 
