@@ -1,7 +1,4 @@
-#ifndef lint
-static char rcsid[] = "@(#$Id: searchdata.cpp,v 1.32 2008-12-19 09:55:36 dockes Exp $ (C) 2006 J.F.Dockes";
-#endif
-/*
+/* Copyright (C) 2006 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -259,9 +256,10 @@ bool SearchData::toNativeQuery(Rcl::Db &db, void *d)
 	     it != vpath.end(); it++){
 	    pvpath.push_back(pathelt_prefix + *it);
 	}
-	xq = Xapian::Query(Xapian::Query::OP_FILTER, xq, 
-			   Xapian::Query(Xapian::Query::OP_PHRASE, 
-					 pvpath.begin(), pvpath.end()));
+	xq = Xapian::Query(m_topdirexcl ? 
+			   Xapian::Query::OP_AND_NOT:Xapian::Query::OP_FILTER, 
+			   xq, Xapian::Query(Xapian::Query::OP_PHRASE, 
+					     pvpath.begin(), pvpath.end()));
     }
 
     *((Xapian::Query *)d) = xq;
@@ -355,6 +353,7 @@ void SearchData::erase() {
     m_query.clear();
     m_filetypes.clear();
     m_topdir.erase();
+    m_topdirexcl = false;
     m_description.erase();
     m_reason.erase();
     m_haveDates = false;
