@@ -1,4 +1,4 @@
-/*
+/* Copyright (C) 2009 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -14,9 +14,6 @@
  *   Free Software Foundation, Inc.,
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
-#ifndef lint
-static char rcsid[] = "@(#$Id: $ (C) 2009 J.F.Dockes";
-#endif
 
 #ifndef TEST_CIRCACHE
 #include "autoconfig.h"
@@ -79,7 +76,7 @@ static bool inflateToDynBuf(void *inp, UINT inlen, void **outpp, UINT *outlenp);
 // Entry header.
 // 3 x 32 bits sizes as hex integers + 1 x 16 bits flag + at least 1 zero
 //                          15 +             3x(9)  + 3 + 1 = 46
-const char *headerformat = "circacheSizes = %x %x %x %hx";
+static const char *headerformat = "circacheSizes = %x %x %x %hx";
 #define CIRCACHE_HEADER_SIZE 64
 
 class EntryHeaderData {
