@@ -1,7 +1,5 @@
-#ifndef lint
-static char rcsid[] = "@(#$Id: wipedir.cpp,v 1.6 2007-12-13 06:58:22 dockes Exp $ (C) 2004 J.F.Dockes";
-#endif
-/*
+/* Copyright (C) 2004 J.F.Dockes
+ *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
